#include "divers.h"
#pragma hrdstop

//##############################################################################
HANDLE OpenFile( AnsiString location, BYTE mode, AnsiString parent )
{
	//move to the location of the file
	if( !SetCurrentDirFromPath( location ) ) return INVALID_HANDLE_VALUE;

	//extract the file+ext part of the location
	AnsiString file = ExtractString( location, FICEXT );

	//open the file
	HANDLE fh = CreateFile( file.c_str(),
							GENERIC_WRITE | GENERIC_READ,
							FILE_SHARE_WRITE | FILE_SHARE_READ,
							NULL,
							( mode == WRITE ) ? CREATE_ALWAYS : OPEN_EXISTING,
							FILE_ATTRIBUTE_NORMAL,
							0);

	//if there is a bug, and a parent has been specified
	if( fh == INVALID_HANDLE_VALUE && parent != "" )
	{ //then we display a messagebox
		AnsiString message = (( mode == WRITE ) ? FIC_NEW : FIC_OP);
		message += location + "!";
		Forbid( parent, message );
	}

	return fh;
}

//##############################################################################
BYTE CheckFile( AnsiString location, int mode )
{
	if( mode == READING || mode == READING_MUTE )
	{ //open a file for reading
		HANDLE fhLect = OpenFile( location, READ );
		if( fhLect == INVALID_HANDLE_VALUE )
		{
			DWORD res = GetLastError();
			//if MUTE, we don't display the error
			if( mode == READING_MUTE ) return res;
			AnsiString message = FIC_OP + location + " !";
			message.cat_printf( " (%d)", res );
			Forbid( "CheckFile", message );
			return res;
		}
		CloseHandle( fhLect ); //it works, but we have to close the file!
		return 0;
	}

	HANDLE fhEcr = OpenFile( location, WRITE ); //open a file for writing

	if( fhEcr == INVALID_HANDLE_VALUE )
	{
		DWORD res = GetLastError();
		if( mode == WRITING_MUTE )	return res;
		if( res == ERROR_FILE_EXISTS )
		{ //special case : the file already exists. Shall we erase it?
			AnsiString message =  FIC_EX1 + location + FIC_EX2;
			if( Warn( WARNING, message ) == IDYES )
			{ //if yes, then there is no error at all
				CloseHandle(fhEcr);
				return 0;
			}
			return res;
		}
		//others cases : an error occured
		AnsiString message = FIC_NEW " !";
		message.cat_printf( " (%d)", res );
		Forbid( "CheckFile", message );
		return res;
	}

	CloseHandle( fhEcr );
	return 0;
}

//##############################################################################
DWORD Read( HANDLE fh, AnsiString &data, char indice )
{
	DWORD nbread, nbreadTotal = 0;
	char tmp;
	data = "";
	ReadFile( fh, &tmp, 1, &nbread, NULL );
	while( tmp != indice && nbread )
	{
		data += tmp;
		nbreadTotal += nbread;
		ReadFile( fh, &tmp, 1, &nbread, NULL );
	};
	return nbreadTotal;
}

//##############################################################################
DWORD Write( HANDLE fh, AnsiString data, int nbr )
{
	DWORD nbwritten;
	int length = ( nbr ) ? nbr : data.Length();
	WriteFile( fh, data.c_str(), length, &nbwritten, NULL );
	return nbwritten;
}

//##############################################################################
void Inform( AnsiString title, AnsiString message )
{
	MessageBox( NULL, message.c_str(), title.c_str(),
							MB_ICONINFORMATION | MB_TASKMODAL );
}
//------------------------------------------------------------------------------
int Ask( AnsiString title, AnsiString message, bool btnAnn )
{
	if( btnAnn )
		return MessageBox( NULL, message.c_str(), title.c_str(),
											 MB_ICONQUESTION | MB_TASKMODAL | MB_YESNOCANCEL );
	else
		return MessageBox( NULL, message.c_str(), title.c_str(),
											 MB_ICONQUESTION | MB_TASKMODAL | MB_YESNO );
}
//------------------------------------------------------------------------------
void Alert( AnsiString title, AnsiString message )
{
	MessageBox( NULL, message.c_str(), title.c_str(),
							MB_ICONWARNING | MB_TASKMODAL );
}
//------------------------------------------------------------------------------
int Warn( AnsiString title, AnsiString message, bool btnAnn )
{
	if( btnAnn )
		return MessageBox( NULL, message.c_str(), title.c_str(),
											 MB_ICONWARNING | MB_TASKMODAL | MB_YESNOCANCEL );
	else
		return MessageBox( NULL, message.c_str(), title.c_str(),
											 MB_ICONWARNING | MB_TASKMODAL | MB_YESNO );
}
//------------------------------------------------------------------------------
void Forbid( AnsiString title, AnsiString message )
{
	MessageBox( NULL, message.c_str(), title.c_str(),
							MB_ICONERROR | MB_TASKMODAL );
}

//##############################################################################
BYTE ReadNumber( HANDLE fh, BYTE &up, BYTE &low )
{
	BYTE octet;

	//first byte
	if( !Read( fh, octet ) ) return -1;
	up = ConvertCharIntoNum( octet );

	//second byte
	Read( fh, octet );
	low = ConvertCharIntoNum( octet );

	return ( ( up * 0x10 ) + low );
}

//##############################################################################
AnsiString ReadSequence( HANDLE fh )
{
	BYTE octet;
	DWORD nbread;
	AnsiString sq = "";

	//read the '=' symbol (see the parent that call this function to understand)
	if( !Read( fh, octet ) ) return "EOF";

	//while we read one byte, and that this byte is not 0x0D (end of line)
	while( Read( fh, octet ) == 1 && octet != 0x0D )
		sq.cat_printf( "%c", octet );

	//read the 0x0A character (an end of line is made with 2 bytes: 0x0A and 0x0D
	Read( fh, octet );

	return sq;
}

//##############################################################################
AnsiString ExtractString( AnsiString string, BYTE code )
{
	switch( code )
	{
		case REP : return IncludeTrailingPathDelimiter( ExtractFilePath( string ) );
		case FICEXT : return ExtractFileName( string );
		case FIC :
		{
			AnsiString f = ExtractFileName( string );
			return f.SetLength( f.LastDelimiter( "." ) - 1 ); //remove the extension
		}
		case EXT : return ExtractFileExt( string );
		default : return string;
	}
}

//##############################################################################
bool SetCurrentDirFromPath( AnsiString dir )
{
	//find out if the directory exists
	bool success = DirectoryExists( ExtractString( dir, REP ) );

	if( success )
		success = SetCurrentDir( ExtractString( dir, REP ) );

	if( !success )
		Forbid( "SetCurrentDirFromPath", ExtractString( dir, REP ) + REP_ERR );

	return success;
}

//##############################################################################
bool FileAlive( AnsiString file )
{
	if( FileExists( file ) ) return true;

	file += FIC_NOT_FOUND;
	Forbid( "FileAlive", file );
	return false;
}

//##############################################################################
BYTE ConvertCharIntoNum( char car )
{
	if( car >= '0' && car <= '9' ) return car - 0x30;
	if( car >= 'A' && car <= 'F' ) return car - 0x37;
	return 99;
}
