#ifndef DiversH
#define DiversH

//Messages
#define WARNING "Attention"
#define FIC_NEW "Impossible de crer le fichier "
#define FIC_OP "Impossible d'ouvrir le fichier "
#define FIC_EX1 "Le fichier "
#define FIC_EX2 " existe dj !\nL'craser ?"
#define FIC_BAD " is not a valid FF7 DAT file!"
#define REP_ERR " n'est pas un rpertoire valide !"
#define FIC_NOT_FOUND " n'a pas t trouv !"

#include <Dialogs.hpp>


/*------------------------------------------------------------------------------
	NAME : OpenFile
	FUNCTION : attempt to open a file
	PARAMS : the file name, the open mode and the name of the parent function in
					 which this function is called (for error messages purpose only)
	RETURN VALUE : handle of the file (returns INVALID_HANDLE_VALUE if error)
	SPEC : if parent is not specified, the function doesn't display any messagebox
				 in case of bug
*/
enum{ READ = 1, WRITE };
HANDLE OpenFile( AnsiString name, BYTE mode, AnsiString parent = "" );


/*------------------------------------------------------------------------------
	NAME : CheckFile
	FUNCTION : see if a file can be read of written
	PARAMS : the file name and the open mode
	RETURN VALUE : error code (0 if no errors)
*/
enum{ READING = 1, //if the function fails, a message will pop up
			READING_MUTE, //if the function fails, nothing happens
			WRITING, //if the file already exists, the user will be asked for the erase
			WRITING_MUTE //if the file already exists, it will automatically be erased
		};
BYTE CheckFile( AnsiString fichier, int mode );


/*------------------------------------------------------------------------------
	NAME : various
	FUNCTION : display various messages, depending on the function
	PARAMS : title of the messagebox, its content, and the way the Cancel button
					 is displayed (if set to true, the button is displayed)
	RETURN VALUE : for the functions that display messagebox with choices, it
								 returns the answer of the user
	SPEC : if the third parameter is not specified, the button is not displayed
*/
void Inform( AnsiString title, AnsiString message ); //display an information
int Ask( AnsiString t, AnsiString m, bool btnAnn = false ); //ask for something
void Alert( AnsiString title, AnsiString message ); // display an alert
int Warn( AnsiString t, AnsiString m, bool btnAnn = false ); //display an alert and let the user make a choice
void Forbid( AnsiString t, AnsiString m ); //display an error, or something critical


/*------------------------------------------------------------------------------
	NAME : ReadNumber
	FUNCTION : read two characters (which are supposed to be digits) in a file
						 and convert them into an hexadecimal number
	PARAMS : handle of the file, upper and lower bytes of the number read
	RETURN VALUE : the number made with the 2 bytes read
*/
BYTE ReadNumber( HANDLE fh, BYTE &up, BYTE &low );


/*------------------------------------------------------------------------------
	NAME : ReadSequence
	FUNCTION : read a sequence of characters in a file, which stops with 0x0D
	PARAMS : handle of the file
	RETURN VALUE : the sequence read
*/
AnsiString ReadSequence( HANDLE fh );


/*------------------------------------------------------------------------------
	NAME : ExtractString
	FUNCTION : extract some specific parts of a string (which has to be the path
						 of a file)
	PARAMS : the string and the code of the part to be extracted
	RETURN VALUE : the extracted part
*/
AnsiString ExtractString( AnsiString string, BYTE code );
enum{ REP = 1, //extract the path, without the file
			FIC,		 //extract the name of the file, without the extension
			FICEXT,  //extract the name of the file with its extention
			EXT 		 //extract only the extension of the file
		};


/*------------------------------------------------------------------------------
	NAME : SetCurrentDirFromPath
	FUNCTION : extract the directory part of a string, and set it as the current
						 directory
	PARAMS : the string
	RETURN VALUE : error code (true if no errors)
*/
bool SetCurrentDirFromPath( AnsiString dir );

/*------------------------------------------------------------------------------
	NAME : FileAlive
	FUNCTION : check if a file exists, and display an error message if not
	PARAMS : location of the file
	RETURN VALUE : result (true if the file exists, otherwise false)
*/
bool FileAlive( AnsiString file );

/*------------------------------------------------------------------------------
	NAME : convertCharIntoNum
	FUNCTION : converts a character into an hexadecimal value
	PARAMS : a character
	RETURN VALUE : the converted number
*/
BYTE ConvertCharIntoNum( char car );

/*------------------------------------------------------------------------------
	NAME : Read
	FUNCTION : read data from a file
	PARAMS : handle of the file, buffer which will receive the data read,
					 number of bytes to read
	RETURN VALUE : number of bytes actually read
	SPEC : if nbr is not specified or null, sizeof( data type) bytes are read
*/
template <class T>
DWORD Read( HANDLE fh, T &data, int nbr = 0 )
{
	DWORD nbread;
	int length = ( nbr ) ? nbr : sizeof( T ); //number of bytes to read
	//try to read length bytes from fh to data, nbread bytes actually read
	ReadFile( fh, &data, length, &nbread, NULL );
	return nbread;
}

/*------------------------------------------------------------------------------
	NAME : Read
	FUNCTION : read a string from a file
	PARAMS : handle of the file, string which will receive the data read,
					 character until which the read will stop
	RETURN VALUE : number of bytes actually read
*/
DWORD Read( HANDLE fh, AnsiString &data, char indice );

/*------------------------------------------------------------------------------
	NAME : Write
	FUNCTION : write data into a file
	PARAMS : handle of the file, buffer which contains the data to be written,
					 number of bytes to write
	RETURN VALUE : number of bytes actually written
	SPEC : if nbr is not specified or null, sizeof( data type) bytes are written
*/
template <class T>
DWORD Write( HANDLE fh, T &data, int nbr = 0 )
{
	DWORD nbwritten;
	int length = ( nbr ) ? nbr : sizeof( T );
	//try to write length bytes of data into fh, nbwritten byte actually written
	WriteFile( fh, &data, length, &nbwritten, NULL );
	return nbwritten;
}

/*------------------------------------------------------------------------------
	NAME : Write
	FUNCTION : write a string into a file
	PARAMS : handle of the file, string to be written, number of bytes to write
	RETURN VALUE : number of bytes actually written
	SPEC : if nbr is not specified or null, the entire string is written
*/
DWORD Write( HANDLE fh, AnsiString data, int nbr = 0 );
#endif
